@@ -1,13 +1,14 @@
 import pkg_resources
 import warnings
+import sys
 
 from sqlalchemy import engine_from_config
 from sqlalchemy import MetaData
 from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.sql.expression import desc
 from sqlalchemy.orm import scoped_session
 from sqlalchemy.orm import sessionmaker
 from zope.sqlalchemy import ZopeTransactionExtension
+from zope.deprecation import deprecation
 from pyramid.authentication import AuthTktAuthenticationPolicy
 from pyramid.authorization import ACLAuthorizationPolicy
 from pyramid.config import Configurator
@@ -16,27 +17,50 @@ from pyramid.threadlocal import get_current_registry
 from pyramid.util import DottedNameResolver
 from pyramid_beaker import session_factory_from_settings
 
-import kotti.patches; kotti.patches
 from kotti.sqla import Base as KottiBase
-from kotti.util import request_cache
+
 
 metadata = MetaData()
 DBSession = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
 Base = declarative_base(cls=KottiBase)
 Base.metadata = metadata
+Base.query = DBSession.query_property()
+TRUE_VALUES = ('1', 'y', 'yes', 't', 'true')
+FALSE_VALUES = ('0', 'n', 'no', 'f', 'false', 'none')
+
+
+# BBB module deprecation
+from kotti import fanstatic
+sys.modules['kotti.static'] = deprecation.deprecated(
+    fanstatic,
+    "The module kotti.static has been moved to kotti.fanstatic as of Kotti "
+    "0.8. Import from there instead.")
+
 
 def authtkt_factory(**settings):
     from kotti.security import list_groups_callback
-    return AuthTktAuthenticationPolicy(
-        secret=settings['kotti.secret2'], callback=list_groups_callback)
+    kwargs = dict(
+        secret=settings['kotti.secret2'],
+        hashalg='sha512',
+        callback=list_groups_callback,
+        )
+    try:
+        return AuthTktAuthenticationPolicy(**kwargs)
+    except TypeError:
+        # BBB with Pyramid < 1.4
+        kwargs.pop('hashalg')
+        return AuthTktAuthenticationPolicy(**kwargs)
+
 
 def acl_factory(**settings):
     return ACLAuthorizationPolicy()
 
+
 def beaker_session_factory(**settings):
     return session_factory_from_settings(settings)
 
-def none_factory(**kwargs): # pragma: no cover
+
+def none_factory(**kwargs):  # pragma: no cover
     return None
 
 # All of these can be set by passing them in the Paste Deploy settings:
@@ -44,20 +68,54 @@ conf_defaults = {
     'kotti.templates.api': 'kotti.views.util.TemplateAPI',
     'kotti.configurators': '',
     'pyramid.includes': '',
-    'kotti.includes': '', # BBB
-    'kotti.base_includes': 'kotti kotti.events kotti.views kotti.views.view kotti.views.edit kotti.views.login kotti.views.file kotti.views.users kotti.views.site_setup kotti.views.slots',
+    'kotti.includes': '',  # BBB
+    'kotti.base_includes': ' '.join([
+        'kotti kotti.events',
+        'kotti.views',
+        'kotti.views.cache',
+        'kotti.views.view',
+        'kotti.views.edit',
+        'kotti.views.edit.actions',
+        'kotti.views.edit.content',
+        'kotti.views.edit.default_views',
+        'kotti.views.login',
+        'kotti.views.file',
+        'kotti.views.image',
+        'kotti.views.users',
+        ]),
+    'kotti.zcml_includes': ' '.join([
+        ]),
     'kotti.asset_overrides': '',
     'kotti.use_tables': '',
     'kotti.root_factory': 'kotti.resources.default_get_root',
     'kotti.populators': 'kotti.populate.populate',
-    'kotti.available_types': 'kotti.resources.Document kotti.resources.File',
+    'kotti.available_types': ' '.join([
+        'kotti.resources.Document',
+        'kotti.resources.File',
+        'kotti.resources.Image',
+        ]),
+    'kotti.search_content': 'kotti.views.util.default_search_content',
     'kotti.authn_policy_factory': 'kotti.authtkt_factory',
     'kotti.authz_policy_factory': 'kotti.acl_factory',
     'kotti.session_factory': 'kotti.beaker_session_factory',
     'kotti.principals_factory': 'kotti.security.principals_factory',
+    'kotti.caching_policy_chooser': (
+        'kotti.views.cache.default_caching_policy_chooser'),
+    'kotti.url_normalizer': 'kotti.url_normalizer.url_normalizer',
+    'kotti.url_normalizer.map_non_ascii_characters': True,
+    'kotti.use_workflow': 'kotti:workflow.zcml',
     'kotti.date_format': 'medium',
     'kotti.datetime_format': 'medium',
     'kotti.time_format': 'medium',
+    'kotti.max_file_size': '10',
+    'kotti.fanstatic.edit_needed': 'kotti.fanstatic.edit_needed',
+    'kotti.fanstatic.view_needed': 'kotti.fanstatic.view_needed',
+    'kotti.static.edit_needed': '',  # BBB
+    'kotti.static.view_needed': '',  # BBB
+    'kotti.alembic_dirs': 'kotti:alembic',
+    'kotti.register': 'False',
+    'kotti.register.group': '',
+    'kotti.register.role': '',
     'pyramid_deform.template_search_path': 'kotti:templates/deform',
     }
 
@@ -68,26 +126,25 @@ conf_dotted = set([
     'kotti.root_factory',
     'kotti.populators',
     'kotti.available_types',
+    'kotti.search_content',
     'kotti.authn_policy_factory',
     'kotti.authz_policy_factory',
     'kotti.session_factory',
     'kotti.principals_factory',
+    'kotti.caching_policy_chooser',
+    'kotti.fanstatic.edit_needed',
+    'kotti.fanstatic.view_needed',
+    'kotti.url_normalizer',
     ])
 
+
 def get_version():
     return pkg_resources.require("Kotti")[0].version
 
-@request_cache(lambda: None)
+
 def get_settings():
-    from kotti.resources import Settings
-    session = DBSession()
-    db_settings = session.query(Settings).order_by(desc(Settings.id)).first()
-    if db_settings is not None:
-        reg_settings = dict(get_current_registry().settings)
-        reg_settings.update(db_settings.data)
-        return reg_settings
-    else:
-        return get_current_registry().settings
+    return get_current_registry().settings
+
 
 def _resolve_dotted(d, keys=conf_dotted):
     for key in keys:
@@ -99,24 +156,29 @@ def _resolve_dotted(d, keys=conf_dotted):
             new_value.append(DottedNameResolver(None).resolve(dottedname))
         d[key] = new_value
 
+
 def main(global_config, **settings):
-    """ This function is a 'paste.app_factory' and returns a WSGI
-    application.
-    """
+    # This function is a 'paste.app_factory' and returns a WSGI
+    # application.
+
+    from kotti.resources import initialize_sql
     config = base_configure(global_config, **settings)
+    engine = engine_from_config(config.registry.settings, 'sqlalchemy.')
+    initialize_sql(engine)
     return config.make_wsgi_app()
 
+
 def base_configure(global_config, **settings):
-    """Resolve dotted names in settings, include plug-ins and create a
-    Configurator.
-    """
-    from kotti.resources import appmaker
+    # Resolve dotted names in settings, include plug-ins and create a
+    # Configurator.
+
+    from kotti.resources import get_root
 
     for key, value in conf_defaults.items():
         settings.setdefault(key, value)
 
     for key, value in settings.items():
-        if isinstance(settings[key], basestring):
+        if key.startswith('kotti') and isinstance(value, basestring):
             settings[key] = unicode(value, 'utf8')
 
     # Allow extending packages to change 'settings' w/ Python:
@@ -124,6 +186,23 @@ def base_configure(global_config, **settings):
     for func in settings['kotti.configurators']:
         func(settings)
 
+    # BBB: Merge ``kotti.static.x`` into kotti.fanstatic.x
+    deprecated_fanstatic_settings = {
+        'kotti.static.view_needed': 'kotti.fanstatic.view_needed',
+        'kotti.static.edit_needed': 'kotti.fanstatic.edit_needed',
+    }
+    for old, new in deprecated_fanstatic_settings.items():
+        if old in settings:
+            if settings[old]:
+                warnings.warn(
+                    "The '%(old)s' setting has been deprecated as of "
+                    "Kotti 0.8 and will be removed in Kotti 1.0.  Use "
+                    "'%(new)s' instead." % {'old': old, 'new': new},
+                    DeprecationWarning)
+                settings.setdefault(new, '')
+                settings[new] += ' ' + settings[old]
+            del settings[old]
+
     _resolve_dotted(settings)
     secret1 = settings['kotti.secret']
     settings.setdefault('kotti.secret2', secret1)
@@ -144,6 +223,9 @@ def base_configure(global_config, **settings):
     config = Configurator(settings=settings)
     config.begin()
 
+    config.hook_zca()
+    config.include('pyramid_zcml')
+
     config.registry.settings['pyramid.includes'] = pyramid_includes
 
     # Include modules listed in 'kotti.base_includes':
@@ -151,17 +233,26 @@ def base_configure(global_config, **settings):
         config.include(module)
     config.commit()
 
-    # Modules in 'pyramid.includes' may override 'kotti.base_includes':
+    # Modules in 'pyramid.includes' and 'kotti.zcml_includes' may
+    # override 'kotti.base_includes':
     if pyramid_includes:
         for module in pyramid_includes.split():
             config.include(module)
+
+    for name in settings['kotti.zcml_includes'].strip().split():
+        config.load_zcml(name)
+
     config.commit()
 
-    engine = engine_from_config(settings, 'sqlalchemy.')
-    config._set_root_factory(appmaker(engine))
+    config._set_root_factory(get_root)
+
+    # add the authenticated user to the request object
+    from kotti.security import get_user
+    config.set_request_property(get_user, name="user", reify=True)
 
     return config
 
+
 def includeme(config):
     import kotti.views.util
 
@@ -188,4 +279,8 @@ def includeme(config):
 
     config.add_translation_dirs('kotti:locale')
 
+    workflow = settings['kotti.use_workflow']
+    if workflow.lower() not in FALSE_VALUES:
+        config.load_zcml(workflow)
+
     return config
