@@ -2,8 +2,12 @@ import hashlib
 import time
 import urllib
 
-from pyramid_mailer.message import Message
+from html2text import HTML2Text
+from pyramid.renderers import render
 from pyramid_mailer.mailer import Mailer
+from pyramid_mailer.message import Message
+from zope.deprecation.deprecation import deprecate
+from zope.deprecation.deprecation import deprecated
 
 from kotti import get_settings
 
@@ -27,13 +31,22 @@ message_templates = {
     'reset-password': dict(
         subject=RESET_PASSWORD_SUBJECT, body=RESET_PASSWORD_BODY),
     }
+deprecated(
+    'message_templates',
+    'message_templates is deprecated as of Kotti 0.6.4.  '
+    'Use the ``email-set-password.pt`` and ``email-reset-password.pt`` '
+    'templates instead.'
+    )
 
 _inject_mailer = []
+
+
 def get_mailer():
     # Consider that we may have persistent settings
     if _inject_mailer:
         return _inject_mailer[0]
-    return Mailer.from_settings(get_settings()) # pragma: no cover
+    return Mailer.from_settings(get_settings())  # pragma: no cover
+
 
 def make_token(user, seconds=None):
     secret = get_settings()['kotti.secret2']
@@ -42,6 +55,7 @@ def make_token(user, seconds=None):
     token = '%s:%s:%s' % (user.name, secret, seconds)
     return '%s:%s' % (hashlib.sha224(token).hexdigest(), seconds)
 
+
 def validate_token(user, token, valid_hrs=24):
     """
       >>> from kotti.testing import setUp, tearDown
@@ -75,6 +89,9 @@ def validate_token(user, token, valid_hrs=24):
             return True
     return False
 
+
+@deprecate('send_set_password is deprecated as of Kotti 0.6.4.  '
+           'Use ``kotti.message.email_set_password`` instead.')
 def send_set_password(user, request, templates='set-password', add_query=None):
     site_title = get_settings()['kotti.site_title']
     token = make_token(user)
@@ -95,9 +112,45 @@ def send_set_password(user, request, templates='set-password', add_query=None):
     if isinstance(templates, str):
         templates = message_templates[templates]
     message = Message(
-        recipients=[u'"%s" <%s>' % (user.title, user.email)], # XXX naive?
+        recipients=[u'"%s" <%s>' % (user.title, user.email)],  # XXX naive?
         subject=templates['subject'] % variables,
         body=templates['body'] % variables,
         )
     mailer = get_mailer()
     mailer.send(message)
+
+
+def email_set_password(user, request,
+                       template_name='kotti:templates/email-set-password.pt',
+                       add_query=None):
+    site_title = get_settings()['kotti.site_title']
+    token = make_token(user)
+    user.confirm_token = unicode(token)
+    set_password_query = {'token': token, 'email': user.email}
+    if add_query:
+        set_password_query.update(add_query)
+    url = '%s/@@set-password?%s' % (
+        request.application_url,
+        urllib.urlencode(set_password_query),
+        )
+    variables = dict(
+        user_title=user.title,
+        site_title=site_title,
+        url=url,
+        )
+
+    text = render(template_name, variables, request)
+    subject, htmlbody = text.strip().split('\n', 1)
+    subject = subject.replace('Subject:', '', 1).strip()
+    html2text = HTML2Text()
+    html2text.body_width = 0
+    textbody = html2text.handle(htmlbody).strip()
+
+    message = Message(
+        recipients=[u'"%s" <%s>' % (user.title, user.email)],  # XXX naive?
+        subject=subject,
+        body=textbody,
+        html=htmlbody,
+        )
+    mailer = get_mailer()
+    mailer.send(message)
