@@ -4,10 +4,10 @@ from datetime import datetime
 from UserDict import DictMixin
 
 import bcrypt
-from sqlalchemy import Column
 from sqlalchemy import Boolean
-from sqlalchemy import Integer
+from sqlalchemy import Column
 from sqlalchemy import DateTime
+from sqlalchemy import Integer
 from sqlalchemy import Unicode
 from sqlalchemy import func
 from sqlalchemy.sql.expression import or_
@@ -20,22 +20,27 @@ from pyramid.security import view_execution_permitted
 from kotti import get_settings
 from kotti import DBSession
 from kotti import Base
+from kotti.sqla import JsonType
 from kotti.util import _
-from kotti.util import JsonType
 from kotti.util import request_cache
 from kotti.util import DontCache
 
+
 def get_principals():
     return get_settings()['kotti.principals_factory'][0]()
 
+
+@request_cache(lambda request: None)
 def get_user(request):
     userid = authenticated_userid(request)
     return get_principals().get(userid)
 
+
 def has_permission(permission, context, request):
     with authz_context(context, request):
         return base_has_permission(permission, context, request)
 
+
 class Principal(Base):
     """A minimal 'Principal' implementation.
 
@@ -85,9 +90,10 @@ class Principal(Base):
         self.creation_date = datetime.now()
         self.last_login_date = None
 
-    def __repr__(self): # pragma: no cover
+    def __repr__(self):  # pragma: no cover
         return '<Principal %r>' % self.name
 
+
 class AbstractPrincipals(object):
     """This class serves as documentation and defines what methods are
     expected from a Principals database.
@@ -169,39 +175,49 @@ USER_MANAGEMENT_ROLES = SHARING_ROLES + ['role:admin']
 _DEFAULT_SHARING_ROLES = SHARING_ROLES[:]
 _DEFAULT_USER_MANAGEMENT_ROLES = USER_MANAGEMENT_ROLES[:]
 
-# This is the ACL that gets set on the site root on creation.
+# This is the ACL that gets set on the site root on creation.  Note
+# that this is only really useful if you're _not_ using workflow.  If
+# you are, then you should look at the permissions in workflow.zcml.
 SITE_ACL = [
     ['Allow', 'system.Everyone', ['view']],
     ['Allow', 'role:viewer', ['view']],
-    ['Allow', 'role:editor', ['view', 'add', 'edit']],
-    ['Allow', 'role:owner', ['view', 'add', 'edit', 'manage']],
+    ['Allow', 'role:editor', ['view', 'add', 'edit', 'state_change']],
+    ['Allow', 'role:owner', ['view', 'add', 'edit', 'manage', 'state_change']],
     ]
 
+
 def set_roles(roles_dict):
     ROLES.clear()
     ROLES.update(roles_dict)
 
+
 def set_sharing_roles(role_names):
     SHARING_ROLES[:] = role_names
 
+
 def set_user_management_roles(role_names):
     USER_MANAGEMENT_ROLES[:] = role_names
 
+
 def reset_roles():
     ROLES.clear()
     ROLES.update(_DEFAULT_ROLES)
 
+
 def reset_sharing_roles():
     SHARING_ROLES[:] = _DEFAULT_SHARING_ROLES
 
+
 def reset_user_management_roles():
     USER_MANAGEMENT_ROLES[:] = _DEFAULT_USER_MANAGEMENT_ROLES
 
+
 def reset():
     reset_roles()
     reset_sharing_roles()
     reset_user_management_roles()
 
+
 class PersistentACLMixin(object):
     def _get_acl(self):
         if self._acl is None:
@@ -216,10 +232,12 @@ class PersistentACLMixin(object):
 
     __acl__ = property(_get_acl, _set_acl, _del_acl)
 
+
 def _cachekey_list_groups_raw(name, context):
     context_id = context is not None and getattr(context, 'id', id(context))
     return (name, context_id)
 
+
 @request_cache(_cachekey_list_groups_raw)
 def list_groups_raw(name, context):
     """A set of group names in given ``context`` for ``name``.
@@ -233,11 +251,12 @@ def list_groups_raw(name, context):
     if isinstance(context, Node):
         return set(
             r[0] for r in DBSession.query(LocalGroup.group_name).filter(
-            LocalGroup.node_id==context.id).filter(
-            LocalGroup.principal_name==name).all()
+            LocalGroup.node_id == context.id).filter(
+            LocalGroup.principal_name == name).all()
             )
     return set()
 
+
 def list_groups(name, context=None):
     """List groups for principal with a given ``name``.
 
@@ -246,13 +265,15 @@ def list_groups(name, context=None):
     """
     return list_groups_ext(name, context)[0]
 
+
 def _cachekey_list_groups_ext(name, context=None, _seen=None, _inherited=None):
     if _seen is not None or _inherited is not None:
         raise DontCache
     else:
-        context_id = context is not None and getattr(context, 'id', id(context))
+        context_id = getattr(context, 'id', id(context))
         return (name, context_id)
 
+
 @request_cache(_cachekey_list_groups_ext)
 def list_groups_ext(name, context=None, _seen=None, _inherited=None):
     name = unicode(name)
@@ -279,7 +300,7 @@ def list_groups_ext(name, context=None, _seen=None, _inherited=None):
             groups.update(group_names)
             if recursing or idx != 0:
                 _inherited.update(group_names)
-    
+
     new_groups = groups - _seen
     _seen.update(new_groups)
     for group_name in new_groups:
@@ -290,23 +311,24 @@ def list_groups_ext(name, context=None, _seen=None, _inherited=None):
 
     return list(groups), list(_inherited)
 
+
 def set_groups(name, context, groups_to_set=()):
     """Set the list of groups for principal with given ``name`` and in
     given ``context``.
     """
     name = unicode(name)
     from kotti.resources import LocalGroup
-    session = DBSession()
-    session.query(LocalGroup).filter(
-        LocalGroup.node_id==context.id).filter(
-        LocalGroup.principal_name==name).delete()
+    DBSession.query(LocalGroup).filter(
+        LocalGroup.node_id == context.id).filter(
+        LocalGroup.principal_name == name).delete()
 
     for group_name in groups_to_set:
-        session.add(LocalGroup(context, name, unicode(group_name)))
+        DBSession.add(LocalGroup(context, name, unicode(group_name)))
+
 
 def list_groups_callback(name, request):
     if not is_user(name):
-        return None # Disallow logging in with groups
+        return None  # Disallow logging in with groups
     if name in get_principals():
         context = request.environ.get(
             'authz_context', getattr(request, 'context', None))
@@ -316,6 +338,7 @@ def list_groups_callback(name, request):
             context = get_root(request)
         return list_groups(name, context)
 
+
 @contextmanager
 def authz_context(context, request):
     before = request.environ.pop('authz_context', None)
@@ -327,16 +350,17 @@ def authz_context(context, request):
         if before is not None:
             request.environ['authz_context'] = before
 
+
 def view_permitted(context, request, name=''):
     with authz_context(context, request):
         return view_execution_permitted(context, request, name)
 
+
 def principals_with_local_roles(context, inherit=True):
     """Return a list of principal names that have local roles in the
     context.
     """
     from resources import LocalGroup
-    session = DBSession()
     principals = set()
     items = [context]
     if inherit:
@@ -344,13 +368,14 @@ def principals_with_local_roles(context, inherit=True):
     for item in items:
         principals.update(
             r[0] for r in
-            session.query(LocalGroup.principal_name).filter(
-                LocalGroup.node_id==item.id).group_by(
+            DBSession.query(LocalGroup.principal_name).filter(
+                LocalGroup.node_id == item.id).group_by(
                 LocalGroup.principal_name).all()
             if not r[0].startswith('role:')
             )
     return list(principals)
 
+
 def map_principals_with_local_roles(context):
     principals = get_principals()
     value = []
@@ -364,11 +389,13 @@ def map_principals_with_local_roles(context):
             value.append((principal, (all, inherited)))
     return sorted(value, key=lambda t: t[0].name)
 
+
 def is_user(principal):
     if not isinstance(principal, basestring):
         principal = principal.name
     return ':' not in principal
 
+
 class Principals(DictMixin):
     """Kotti's default principal database.
 
@@ -382,33 +409,29 @@ class Principals(DictMixin):
     @request_cache(lambda self, name: name)
     def __getitem__(self, name):
         name = unicode(name)
-        session = DBSession()
         try:
-            return session.query(
-                self.factory).filter(self.factory.name==name).one()
+            return DBSession.query(
+                self.factory).filter(self.factory.name == name).one()
         except NoResultFound:
             raise KeyError(name)
 
     def __setitem__(self, name, principal):
         name = unicode(name)
-        session = DBSession()
         if isinstance(principal, dict):
             principal = self.factory(**principal)
-        session.add(principal)
+        DBSession.add(principal)
 
     def __delitem__(self, name):
         name = unicode(name)
-        session = DBSession()
         try:
-            principal = session.query(
-                self.factory).filter(self.factory.name==name).one()
-            session.delete(principal)
+            principal = DBSession.query(
+                self.factory).filter(self.factory.name == name).one()
+            DBSession.delete(principal)
         except NoResultFound:
             raise KeyError(name)
 
     def iterkeys(self):
-        session = DBSession()
-        for (principal_name,) in session.query(self.factory.name):
+        for (principal_name,) in DBSession.query(self.factory.name):
             yield principal_name
 
     def keys(self):
@@ -427,16 +450,17 @@ class Principals(DictMixin):
             else:
                 filters.append(col == value)
 
-        session = DBSession()
-        query = session.query(self.factory)
+        query = DBSession.query(self.factory)
         query = query.filter(or_(*filters))
         return query
 
     log_rounds = 10
+
     def hash_password(self, password, hashed=None):
         if hashed is None:
             hashed = bcrypt.gensalt(self.log_rounds)
-        return unicode(bcrypt.hashpw(password, hashed))
+        return unicode(
+            bcrypt.hashpw(password.encode('utf-8'), hashed.encode('utf-8')))
 
     def validate_password(self, clear, hashed):
         try:
@@ -444,5 +468,6 @@ class Principals(DictMixin):
         except ValueError:
             return False
 
+
 def principals_factory():
     return Principals()
