@@ -1,109 +1,87 @@
-from unittest import TestCase
-
 from mock import patch
+from pytest import raises
 from pyramid.authentication import CallbackAuthenticationPolicy
 
 from kotti.testing import DummyRequest
-from kotti.testing import UnitTestBase
 
-class TestGroups(UnitTestBase):
-    def test_root_default(self):
-        from kotti.resources import get_root
+
+class TestGroups:
+    def test_root_default(self, db_session, root):
         from kotti.security import list_groups
         from kotti.security import list_groups_raw
 
-        root = get_root()
-        self.assertEqual(list_groups('admin', root), ['role:admin'])
-        self.assertEqual(list_groups_raw(u'admin', root), set([]))
+        assert list_groups('admin', root) == ['role:admin']
+        assert list_groups_raw(u'admin', root) == set([])
 
-    def test_empty(self):
-        from kotti.resources import get_root
+    def test_empty(self, db_session, root):
         from kotti.security import list_groups
 
-        root = get_root()
-        self.assertEqual(list_groups('bob', root), [])
+        assert list_groups('bob', root) == []
 
-    def test_simple(self):
-        from kotti.resources import get_root
+    def test_simple(self, db_session, root):
         from kotti.security import list_groups
         from kotti.security import list_groups_raw
         from kotti.security import set_groups
 
-        root = get_root()
         set_groups('bob', root, ['role:editor'])
-        self.assertEqual(
-            list_groups('bob', root), ['role:editor'])
-        self.assertEqual(
-            list_groups_raw(u'bob', root), set(['role:editor']))
+        assert list_groups('bob', root) == ['role:editor']
+        assert list_groups_raw(u'bob', root) == set(['role:editor'])
 
     def test_not_a_node(self):
         from kotti.security import list_groups_raw
-        
-        self.assertEqual(list_groups_raw(u'bob', object()), set())
 
-    def test_overwrite_and_delete(self):
+        assert list_groups_raw(u'bob', object()) == set()
+
+    def test_overwrite_and_delete(self, db_session, root):
         from kotti.resources import get_root
         from kotti.security import list_groups
         from kotti.security import list_groups_raw
         from kotti.security import set_groups
 
-        root = get_root()
         set_groups('bob', root, ['role:editor'])
-        self.assertEqual(
-            list_groups('bob', root), ['role:editor'])
-        self.assertEqual(
-            list_groups_raw(u'bob', root), set(['role:editor']))
+        assert list_groups('bob', root) == ['role:editor']
+        assert list_groups_raw(u'bob', root) == set(['role:editor'])
 
         set_groups('bob', root, ['role:admin'])
-        self.assertEqual(
-            list_groups('bob', root), ['role:admin'])
-        self.assertEqual(
-            list_groups_raw(u'bob', root), set(['role:admin']))
+        assert list_groups('bob', root) == ['role:admin']
+        assert list_groups_raw(u'bob', root) == set(['role:admin'])
 
         set_groups('bob', root)
-        self.assertEqual(
-            list_groups('bob', root), [])
-        self.assertTrue(get_root() is root)
+        assert list_groups('bob', root) == []
+        assert get_root() is root
 
-    def test_inherit(self):
-        from kotti import DBSession
-        from kotti.resources import get_root
+    def test_inherit(self, db_session, root):
         from kotti.resources import Node
         from kotti.security import list_groups
         from kotti.security import list_groups_raw
         from kotti.security import set_groups
 
-        root = get_root()
         child = root[u'child'] = Node()
-        DBSession.flush()
+        db_session.flush()
 
-        self.assertEqual(list_groups('bob', child), [])
+        assert list_groups('bob', child) == []
         set_groups('bob', root, ['role:editor'])
-        self.assertEqual(list_groups('bob', child), ['role:editor'])
+        assert list_groups('bob', child) == ['role:editor']
 
         # Groups from the child are added:
         set_groups('bob', child, ['group:somegroup'])
-        self.assertEqual(
-            set(list_groups('bob', child)),
+        assert (
+            set(list_groups('bob', child)) ==
             set(['group:somegroup', 'role:editor'])
             )
 
         # We can ask to list only those groups that are defined locally:
-        self.assertEqual(
-            list_groups_raw(u'bob', child), set(['group:somegroup']))
+        assert list_groups_raw(u'bob', child) == set(['group:somegroup'])
 
     @staticmethod
-    def add_some_groups():
-        from kotti import DBSession
-        from kotti.resources import get_root
+    def add_some_groups(db_session, root):
         from kotti.resources import Node
         from kotti.security import set_groups
 
-        root = get_root()
         child = root[u'child'] = Node()
         grandchild = child[u'grandchild'] = Node()
-        DBSession.flush()
-        
+        db_session.flush()
+
         # root:
         #   bob               -> group:bobsgroup
         #   frank             -> group:franksgroup
@@ -130,84 +108,60 @@ class TestGroups(UnitTestBase):
         set_groups('group:franksgroup', grandchild,
                    ['role:owner', 'group:bobsgroup'])
 
-    def test_nested_groups(self):
-        from kotti.resources import get_root
+    def test_nested_groups(self, db_session, root):
         from kotti.security import list_groups
         from kotti.security import list_groups_ext
 
-        self.add_some_groups()
-        root = get_root()
+        self.add_some_groups(db_session, root)
         child = root[u'child']
         grandchild = child[u'grandchild']
 
         # Check bob's groups on every level:
-        self.assertEqual(list_groups('bob', root), ['group:bobsgroup'])
-        self.assertEqual(
-            set(list_groups('bob', child)),
-            set(['group:bobsgroup', 'group:franksgroup', 'role:editor'])
-            )
-        self.assertEqual(
-            set(list_groups('bob', grandchild)),
+        assert list_groups('bob', root) == ['group:bobsgroup']
+        assert (set(list_groups('bob', child)) ==
+            set(['group:bobsgroup', 'group:franksgroup', 'role:editor']))
+        assert (set(list_groups('bob', grandchild)) ==
             set(['group:bobsgroup', 'group:franksgroup', 'role:editor',
-                 'role:owner'])
-            )
+                 'role:owner']))
 
         # Check group:franksgroup groups on every level:
-        self.assertEqual(
-            set(list_groups('frank', root)),
-            set(['group:franksgroup', 'role:editor'])
-            )
-        self.assertEqual(
-            set(list_groups('frank', child)),
-            set(['group:franksgroup', 'role:editor'])
-            )
-        self.assertEqual(
-            set(list_groups('frank', grandchild)),
+        assert (set(list_groups('frank', root)) ==
+            set(['group:franksgroup', 'role:editor']))
+        assert (set(list_groups('frank', child)) ==
+            set(['group:franksgroup', 'role:editor']))
+        assert (set(list_groups('frank', grandchild)) ==
             set(['group:franksgroup', 'role:editor', 'role:owner',
-                 'group:bobsgroup'])
-            )
+                 'group:bobsgroup']))
 
         # Sometimes it's useful to know which of the groups were
         # inherited, that's what 'list_groups_ext' is for:
         groups, inherited = list_groups_ext('bob', root)
-        self.assertEqual(groups, ['group:bobsgroup'])
-        self.assertEqual(inherited, [])
+        assert groups == ['group:bobsgroup']
+        assert inherited == []
 
         groups, inherited = list_groups_ext('bob', child)
-        self.assertEqual(
-            set(groups),
-            set(['group:bobsgroup', 'group:franksgroup', 'role:editor'])
-            )
-        self.assertEqual(
-            set(inherited),
-            set(['group:bobsgroup', 'group:franksgroup', 'role:editor'])
-            )
+        assert (set(groups) ==
+            set(['group:bobsgroup', 'group:franksgroup', 'role:editor']))
+        assert (set(inherited) ==
+            set(['group:bobsgroup', 'group:franksgroup', 'role:editor']))
 
         groups, inherited = list_groups_ext('group:bobsgroup', child)
-        self.assertEqual(
-            set(groups),
-            set(['group:franksgroup', 'role:editor'])
-            )
-        self.assertEqual(inherited, ['role:editor'])
+        assert set(groups) == set(['group:franksgroup', 'role:editor'])
+        assert inherited == ['role:editor']
 
         groups, inherited = list_groups_ext('group:franksgroup', grandchild)
-        self.assertEqual(
-            set(groups),
-            set(['group:bobsgroup', 'role:owner', 'role:editor'])
-            )
-        self.assertEqual(inherited, ['role:editor'])
+        assert (set(groups) ==
+            set(['group:bobsgroup', 'role:owner', 'role:editor']))
+        assert inherited == ['role:editor']
 
-    def test_works_with_auth(self):
-        from kotti import DBSession
-        from kotti.resources import get_root
+    def test_works_with_auth(self, db_session, root):
         from kotti.resources import Node
         from kotti.security import get_principals
         from kotti.security import list_groups_callback
         from kotti.security import set_groups
 
-        root = get_root()
         child = root[u'child'] = Node()
-        DBSession.flush()
+        db_session.flush()
 
         request = DummyRequest()
         auth = CallbackAuthenticationPolicy()
@@ -215,22 +169,22 @@ class TestGroups(UnitTestBase):
         auth.callback = list_groups_callback
 
         request.context = root
-        self.assertEqual( # user doesn't exist yet
-            auth.effective_principals(request),
+        assert (  # user doesn't exist yet
+            auth.effective_principals(request) ==
             ['system.Everyone']
             )
 
         get_principals()[u'bob'] = dict(name=u'bob')
-        self.assertEqual(
-            auth.effective_principals(request),
+        assert (
+            auth.effective_principals(request) ==
             ['system.Everyone', 'system.Authenticated', 'bob']
             )
 
         # Define that bob belongs to bobsgroup on the root level:
         set_groups('bob', root, ['group:bobsgroup'])
         request.context = child
-        self.assertEqual(
-            set(auth.effective_principals(request)), set([
+        assert (
+            set(auth.effective_principals(request)) == set([
                 'system.Everyone', 'system.Authenticated',
                 'bob', 'group:bobsgroup'
                 ])
@@ -239,8 +193,8 @@ class TestGroups(UnitTestBase):
         # define that bob belongs to franksgroup in the user db:
         get_principals()[u'bob'].groups = [u'group:franksgroup']
         set_groups('group:franksgroup', child, ['group:anothergroup'])
-        self.assertEqual(
-            set(auth.effective_principals(request)), set([
+        assert (
+            set(auth.effective_principals(request)) == set([
                 'system.Everyone', 'system.Authenticated',
                 'bob', 'group:bobsgroup', 'group:franksgroup',
                 'group:anothergroup',
@@ -254,15 +208,15 @@ class TestGroups(UnitTestBase):
             title=u"Frank's group",
             groups=[u'group:funnygroup', u'group:bobsgroup'],
             )
-        self.assertEqual(
-            set(auth.effective_principals(request)), set([
+        assert (
+            set(auth.effective_principals(request)) == set([
                 'system.Everyone', 'system.Authenticated',
                 'bob', 'group:bobsgroup', 'group:franksgroup',
                 'group:anothergroup', 'group:funnygroup',
                 ])
             )
 
-    def test_list_groups_callback_with_groups(self):
+    def test_list_groups_callback_with_groups(self, db_session):
         from kotti.security import list_groups_callback
         from kotti.security import get_principals
 
@@ -270,132 +224,117 @@ class TestGroups(UnitTestBase):
         # we're not allowed to authenticate with a group id:
         get_principals()[u'bob'] = dict(name=u'bob')
         get_principals()[u'group:bobsgroup'] = dict(name=u'group:bobsgroup')
-        
-        request = DummyRequest()
-        self.assertEqual(
-            list_groups_callback(u'bob', request), [])
-        self.assertEqual(
-            list_groups_callback(u'group:bobsgroup', request), None)
 
-    def test_principals_with_local_roles(self):
-        from kotti import DBSession
-        from kotti.resources import get_root
+        request = DummyRequest()
+        assert list_groups_callback(u'bob', request) == []
+        assert list_groups_callback(u'group:bobsgroup', request) is None
+
+    def test_principals_with_local_roles(self, db_session, root):
         from kotti.resources import Node
         from kotti.security import map_principals_with_local_roles
         from kotti.security import principals_with_local_roles
         from kotti.security import set_groups
 
-        root = get_root()
         child = root[u'child'] = Node()
-        DBSession.flush()
+        db_session.flush()
 
-        self.assertEqual(principals_with_local_roles(root), [])
-        self.assertEqual(principals_with_local_roles(child), [])
-        self.assertEqual(map_principals_with_local_roles(root), [])
-        self.assertEqual(map_principals_with_local_roles(child), [])
+        assert principals_with_local_roles(root) == []
+        assert principals_with_local_roles(child) == []
+        assert map_principals_with_local_roles(root) == []
+        assert map_principals_with_local_roles(child) == []
 
         set_groups('group:bobsgroup', child, ['role:editor'])
         set_groups('bob', root, ['group:bobsgroup'])
         set_groups('group:franksgroup', root, ['role:editor'])
 
-        self.assertEqual(
-            set(principals_with_local_roles(child)),
+        assert (
+            set(principals_with_local_roles(child)) ==
             set(['bob', 'group:bobsgroup', 'group:franksgroup'])
             )
-        self.assertEqual(
-            set(principals_with_local_roles(child, inherit=False)),
+        assert (
+            set(principals_with_local_roles(child, inherit=False)) ==
             set(['group:bobsgroup'])
             )
-        self.assertEqual(
-            set(principals_with_local_roles(root)),
+        assert (
+            set(principals_with_local_roles(root)) ==
             set(['bob', 'group:franksgroup'])
             )
 
-    def test_copy_local_groups(self):
-        from kotti import DBSession
-        from kotti.resources import get_root
+    def test_copy_local_groups(self, db_session, root):
         from kotti.security import principals_with_local_roles
         from kotti.security import set_groups
 
-        self.test_principals_with_local_roles()
-        root = get_root()
+        self.test_principals_with_local_roles(db_session, root)
         child = root[u'child']
-        self.assertEqual(
-            set(principals_with_local_roles(child)),
+        assert (
+            set(principals_with_local_roles(child)) ==
             set(['bob', 'group:bobsgroup', 'group:franksgroup'])
             )
 
-        # We make a copy of 'child', and we expect the local roles to
-        # be copied over:
+        # We make a copy of 'child', and we expect the local roles set
+        # on 'child' to not be copied over:
         child2 = root['child2'] = child.copy()
-        DBSession.flush()
-        self.assertEqual(
-            set(principals_with_local_roles(child2)),
-            set(['bob', 'group:bobsgroup', 'group:franksgroup'])
-            )
-        self.assertEqual(len(principals_with_local_roles(child)), 3)
+        db_session.flush()
+        assert (
+            set(principals_with_local_roles(child2)) ==
+            set([u'bob', u'group:franksgroup']))
+        assert len(principals_with_local_roles(child)) == 3
 
         # When we now change the local roles of 'child', the copy is
         # unaffected:
         set_groups('group:bobsgroup', child, [])
-        self.assertEqual(len(principals_with_local_roles(child)), 2)
-        self.assertEqual(len(principals_with_local_roles(child2)), 3)
+        assert len(principals_with_local_roles(child)) == 2
+        assert len(principals_with_local_roles(child2)) == 2
 
-    def test_map_principals_with_local_roles(self):
-        from kotti.resources import get_root
+    def test_map_principals_with_local_roles(self, db_session, root):
         from kotti.security import get_principals
         from kotti.security import map_principals_with_local_roles
 
-        self.test_principals_with_local_roles()
-        root = get_root()
+        self.test_principals_with_local_roles(db_session, root)
         child = root[u'child']
         P = get_principals()
 
         # No users are defined in P, thus we get the empty list:
-        self.assertEqual(map_principals_with_local_roles(root), [])
+        assert map_principals_with_local_roles(root) == []
 
         P['bob'] = {'name': u'bob'}
         P['group:bobsgroup'] = {'name': u'group:bobsgroup'}
 
         value = map_principals_with_local_roles(root)
-        self.assertEqual(len(value), 1)
+        assert len(value) == 1
         bob, (bob_all, bob_inherited) = value[0]
-        self.assertEqual(bob_all, ['group:bobsgroup'])
-        self.assertEqual(bob_inherited, [])
+        assert bob_all == ['group:bobsgroup']
+        assert bob_inherited == []
 
         value = map_principals_with_local_roles(child)
-        self.assertEqual(len(value), 2)
+        assert len(value) == 2
         bob, (bob_all, bob_inherited) = value[0]
         bobsgroup, (bobsgroup_all, bobsgroup_inherited) = value[1]
-        self.assertEqual(set(bob_all),
-                         set(['group:bobsgroup', 'role:editor']))
-        self.assertEqual(set(bob_inherited),
-                         set(['group:bobsgroup', 'role:editor']))
-        self.assertEqual(bobsgroup_all, ['role:editor'])
-        self.assertEqual(bobsgroup_inherited, [])
+        assert (set(bob_all) == set(['group:bobsgroup', 'role:editor']))
+        assert (set(bob_inherited) == set(['group:bobsgroup', 'role:editor']))
+        assert bobsgroup_all == ['role:editor']
+        assert bobsgroup_inherited == []
 
-    def test_local_roles_db_cascade(self):
-        from kotti import DBSession
-        from kotti.resources import get_root
+    def test_local_roles_db_cascade(self, db_session, root):
         from kotti.resources import LocalGroup
         from kotti.resources import Node
         from kotti.security import set_groups
 
-        root = get_root()
         child = root[u'child'] = Node()
-        DBSession.flush()
+        db_session.flush()
 
         # We set a local group on child and delete child.  We then
         # expect the LocalGroup entry to have been deleted from the
         # database:
-        self.assertEqual(DBSession.query(LocalGroup).count(), 0)
+        assert db_session.query(LocalGroup).count() == 0
         set_groups('group:bobsgroup', child, ['role:editor'])
-        self.assertEqual(DBSession.query(LocalGroup).count(), 1)
+        assert db_session.query(LocalGroup).count() == 1
         del root[u'child']
-        DBSession.flush()
-        self.assertEqual(DBSession.query(LocalGroup).count(), 0)
+        db_session.flush()
+        assert db_session.query(LocalGroup).count() == 0
 
-class TestPrincipals(UnitTestBase):
+
+class TestPrincipals:
     def get_principals(self):
         from kotti.security import get_principals
         return get_principals()
@@ -410,37 +349,44 @@ class TestPrincipals(UnitTestBase):
             groups=[u'group:bobsgroup'],
             )
         return users[u'bob']
-    
+
     def _assert_is_bob(self, bob):
-        self.assertEqual(bob.name, u'bob')
-        self.assertEqual(bob.title, u'Bob Dabolina')
-        self.assertEqual(bob.groups, [u'group:bobsgroup'])
+        assert bob.name == u'bob'
+        assert bob.title == u'Bob Dabolina'
+        assert bob.groups == [u'group:bobsgroup']
 
-    def test_default_admin(self):
+    def test_hash_password_non_ascii(self, db_session):
+        self.get_principals().hash_password(u'\xd6TEst')
+
+    def test_default_admin(self, db_session):
         admin = self.get_principals()[u'admin']
-        self.assertTrue(
-            self.get_principals().validate_password(u'secret', admin.password))
-        self.assertEqual(admin.groups, [u'role:admin'])
+        assert self.get_principals().validate_password(
+            u'secret', admin.password)
+        assert admin.groups == [u'role:admin']
 
-    def test_users_empty(self):
+    def test_users_empty(self, db_session):
         users = self.get_principals()
-        self.assertRaises(KeyError, users.__getitem__, u'bob')
-        self.assertRaises(KeyError, users.__delitem__, u'bob')
-        self.assertEqual(users.keys(), [u'admin'])
+        with raises(KeyError):
+            users[u'bob']
+        with raises(KeyError):
+            del users[u'bob']
+        assert users.keys() == [u'admin']
 
-    def test_users_add_and_remove(self):
+    def test_users_add_and_remove(self, db_session):
         self.make_bob()
         users = self.get_principals()
         self._assert_is_bob(users[u'bob'])
-        self.assertEqual(set(users.keys()), set([u'admin', u'bob']))
+        assert set(users.keys()) == set([u'admin', u'bob'])
 
         del users['bob']
-        self.assertRaises(KeyError, users.__getitem__, u'bob')
-        self.assertRaises(KeyError, users.__delitem__, u'bob')
+        with raises(KeyError):
+            users[u'bob']
+        with raises(KeyError):
+            del users[u'bob']
 
-    def test_users_search(self):
+    def test_users_search(self, db_session):
         users = self.get_principals()
-        self.assertEqual(list(users.search(name=u"*Bob*")), [])
+        assert list(users.search(name=u"*Bob*")) == []
         self.make_bob()
         [bob] = list(users.search(name=u"bob"))
         self._assert_is_bob(bob)
@@ -448,128 +394,165 @@ class TestPrincipals(UnitTestBase):
         self._assert_is_bob(bob)
         [bob] = list(users.search(name=u"*Bob*", title=u"*Frank*"))
         self._assert_is_bob(bob)
-        self.assertRaises(AttributeError,
-                          users.search, name=u"bob", foo=u"bar")
-        self.assertEqual(list(users.search()), [])
+        with raises(AttributeError):
+            users.search(name=u"bob", foo=u"bar")
+        assert list(users.search()) == []
 
-    def test_groups_from_users(self):
-        from kotti import DBSession
-        from kotti.resources import get_root
+    def test_groups_from_users(self, db_session, root):
         from kotti.resources import Node
         from kotti.security import list_groups
         from kotti.security import set_groups
 
         self.make_bob()
-        root = get_root()
         child = root[u'child'] = Node()
-        DBSession.flush()
+        db_session.flush()
 
-        self.assertEqual(list_groups('bob', root), ['group:bobsgroup'])
+        assert list_groups('bob', root) == ['group:bobsgroup']
 
         set_groups('group:bobsgroup', root, ['role:editor'])
         set_groups('role:editor', child, ['group:foogroup'])
 
-        self.assertEqual(
-            set(list_groups('bob', root)),
-            set(['group:bobsgroup', 'role:editor'])
-            )
-        self.assertEqual(
-            set(list_groups('bob', child)),
-            set(['group:bobsgroup', 'role:editor', 'group:foogroup'])
-            )
+        assert (set(list_groups('bob', root)) ==
+            set(['group:bobsgroup', 'role:editor']))
+        assert (set(list_groups('bob', child)) ==
+            set(['group:bobsgroup', 'role:editor', 'group:foogroup']))
 
-    def test_is_user(self):
+    def test_is_user(self, db_session):
         from kotti.security import is_user
-        
-        bob = self.make_bob()
-        self.assertEqual(is_user(bob), True)
-        bob.name = u'group:bobsgroup'
-        self.assertEqual(is_user(bob), False)
 
-    def test_hash_and_validate_password(self):
+        bob = self.make_bob()
+        assert is_user(bob) is True
+        bob.name = u'group:bobsgroup'
+        assert is_user(bob) is False
+
+    def test_hash_and_validate_password(self, db_session):
         password = u"secret"
         principals = self.get_principals()
         hashed = principals.hash_password(password)
-        self.assertTrue(principals.validate_password(password, hashed))
-        self.assertFalse(principals.validate_password(u"foo", hashed))
+        assert principals.validate_password(password, hashed)
+        assert principals.validate_password(u"foo", hashed) is False
 
-    def test_bobs_hashed_password(self):
+    def test_bobs_hashed_password(self, db_session):
         bob = self.make_bob()
         principals = self.get_principals()
-        self.assertTrue(principals.validate_password(u"secret", bob.password))
+        assert principals.validate_password(u"secret", bob.password)
 
         # When we set the 'password' attribute directly, we have to do
         # the hashing ourselves.  This won't work:
         bob.password = u'anothersecret'
-        self.assertFalse(
-            principals.validate_password(u"anothersecret", bob.password))
+        assert principals.validate_password(
+            u"anothersecret", bob.password) is False
 
         # This will:
         bob.password = principals.hash_password(u"anothersecret")
-        self.assertTrue(
-            principals.validate_password(u"anothersecret", bob.password))
+        assert principals.validate_password(u"anothersecret", bob.password)
 
-    def test_active(self):
+    def test_active(self, db_session):
         bob = self.make_bob()
-        self.assertEqual(bob.active, True)
+        assert bob.active is True
         bob.active = False
-        self.assertEqual(bob.active, False)
+        assert bob.active is False
 
-    def test_login(self):
+    def test_reset_password(self, db_session):
+        from kotti.views.login import login
+
+        request = DummyRequest()
+        self.make_bob()
+        request.params['reset-password'] = u'on'
+        request.params['login'] = u'bob'
+        request.params['password'] = u'secret'
+        with patch(
+            'kotti.views.login.email_set_password') as email_set_password:
+            login(None, request)
+        assert (request.session.pop_flash('success') == [
+            u"You should receive an email with a link to reset your "
+            u"password momentarily."])
+        assert email_set_password.call_count == 1
+
+    def test_reset_password_inactive_user(self, db_session):
+        from kotti.views.login import login
+
+        request = DummyRequest()
+        self.make_bob().active = False
+        request.params['reset-password'] = u'on'
+        request.params['login'] = u'bob'
+        request.params['password'] = u'secret'
+        with patch(
+            'kotti.views.login.email_set_password') as email_set_password:
+            login(None, request)
+        assert (request.session.pop_flash('error') ==
+            [u"That username or email is not known to us."])
+        assert email_set_password.call_count == 0
+
+    def test_login(self, db_session):
         from kotti.views.login import login
         request = DummyRequest()
 
         # No login attempt:
         result = login(None, request)
-        self.assert_(isinstance(result, dict))
-        self.assertEqual(request.session.pop_flash('success'), [])
-        self.assertEqual(request.session.pop_flash('error'), [])
+        assert isinstance(result, dict)
+        assert request.session.pop_flash('success') == []
+        assert request.session.pop_flash('error') == []
 
         # Attempt to log in before Bob exists:
         request.params['submit'] = u'on'
         request.params['login'] = u'bob'
         request.params['password'] = u'secret'
         result = login(None, request)
-        self.assert_(isinstance(result, dict))
-        self.assertEqual(request.session.pop_flash('success'), [])
-        self.assertEqual(request.session.pop_flash('error'),
-                         [u'Login failed.'])
+        assert isinstance(result, dict)
+        assert request.session.pop_flash('success') == []
+        assert (request.session.pop_flash('error') == [u'Login failed.'])
 
         # Make Bob and do it again:
         bob = self.make_bob()
-        self.assertEqual(bob.last_login_date, None)
+        assert bob.last_login_date is None
         result = login(None, request)
-        self.assertEqual(result.status, '302 Found')
-        self.assertEqual(request.session.pop_flash('success'),
-                         [u'Welcome, Bob Dabolina!'])
+        assert result.status == '302 Found'
+        assert (
+            [request.session.pop_flash('success')[0].interpolate()] ==
+            [u'Welcome, Bob Dabolina!'])
         last_login_date = bob.last_login_date
-        self.assertNotEqual(last_login_date, None)
-        self.assertEqual(request.session.pop_flash('error'), [])
+        assert last_login_date is not None
+        assert request.session.pop_flash('error') == []
 
         # Log in with email:
         request.params['login'] = u'bob@dabolina.com'
         result = login(None, request)
-        self.assertEqual(result.status, '302 Found')
-        self.assertEqual(request.session.pop_flash('success'),
-                         [u'Welcome, Bob Dabolina!'])
-        self.assertTrue(last_login_date < bob.last_login_date)
+        assert result.status == '302 Found'
+        assert (
+            [request.session.pop_flash('success')[0].interpolate()] ==
+            [u'Welcome, Bob Dabolina!'])
+        assert last_login_date < bob.last_login_date
 
         # Deactive Bob, logging in is no longer possible:
         bob.active = False
         result = login(None, request)
-        self.assert_(isinstance(result, dict))
-        self.assertEqual(request.session.pop_flash('error'),
-                         [u'Login failed.'])
+        assert isinstance(result, dict)
+        assert (request.session.pop_flash('error') == [u'Login failed.'])
 
         # If Bob has a 'confirm_token' set, logging in is still possible:
         bob.active = True
         bob.confirm_token = u'token'
         result = login(None, request)
-        self.assertEqual(result.status, '302 Found')
-        self.assertEqual(request.session.pop_flash('success'),
-                         [u'Welcome, Bob Dabolina!'])
+        assert result.status == '302 Found'
+        assert (
+            [request.session.pop_flash('success')[0].interpolate()] ==
+            [u'Welcome, Bob Dabolina!'])
 
-class TestAuthzContextManager(TestCase):
+    def test_login_with_email_remembers(self, db_session):
+        from kotti.views.login import login
+        request = DummyRequest()
+
+        self.make_bob()
+        request.params['submit'] = u'on'
+        request.params['login'] = u'bob@dabolina.com'
+        request.params['password'] = u'secret'
+        with patch('kotti.views.login.remember') as remember:
+            login(None, request)
+            remember.assert_called_with(request, u'bob')
+
+
+class TestAuthzContextManager:
     def test_basic(self):
         from kotti.security import authz_context
 
@@ -599,7 +582,8 @@ class TestAuthzContextManager(TestCase):
         except ValueError:
             assert request.environ['authz_context'] == context2
 
-class TestHasPermission(TestCase):
+
+class TestHasPermission:
     def test_basic(self):
         from kotti.security import has_permission
 
@@ -616,7 +600,8 @@ class TestHasPermission(TestCase):
 
         assert args == [(permission, context, request)]
 
-class TestRolesSetters(UnitTestBase):
+
+class TestRolesSetters:
     def test_set_roles(self):
         from kotti.security import ROLES
         from kotti.security import set_roles
