@@ -1,9 +1,19 @@
+# -*- coding: utf-8 -*-
+
+"""
+Inheritance Diagram
+-------------------
+
+.. inheritance-diagram:: kotti.util
+"""
+
 import re
 import urllib
 
-from plone.i18n.normalizer import urlnormalizer
+from docopt import docopt
 from pyramid.i18n import get_locale_name
 from pyramid.i18n import TranslationStringFactory
+from pyramid.paster import bootstrap
 from pyramid.threadlocal import get_current_request
 from pyramid.url import resource_url
 from repoze.lru import LRUCache
@@ -11,6 +21,7 @@ from zope.deprecation import deprecated
 
 _ = TranslationStringFactory('Kotti')
 
+
 class ViewLink(object):
     def __init__(self, path, title=None):
         self.path = path
@@ -21,25 +32,34 @@ class ViewLink(object):
     def url(self, context, request):
         return resource_url(context, request) + '@@' + self.path
 
-    def selected(self, context, request):
-        return urllib.unquote(request.url).startswith(
-            self.url(context, request))
-
     def permitted(self, context, request):
         from kotti.security import view_permitted
         return view_permitted(context, request, self.path)
 
+    def selected(self, context, request):
+        return urllib.unquote(request.url).startswith(
+            self.url(context, request))
+
     def __eq__(self, other):
         return isinstance(other, ViewLink) and repr(self) == repr(other)
 
     def __repr__(self):
         return "ViewLink(%r, %r)" % (self.path, self.title)
 
+
+class ActionButton(ViewLink):
+    def __init__(self, path, title=None, no_children=False, css_class=u"btn"):
+        super(ActionButton, self).__init__(path, title)
+        self.no_children = no_children
+        self.css_class = css_class
+
+
 class DontCache(Exception):
     pass
 
 _CACHE_ATTR = 'kotti_cache'
 
+
 def request_container():
     request = get_current_request()
     if request is None:
@@ -50,8 +70,10 @@ def request_container():
         setattr(request, _CACHE_ATTR, cache)
     return cache
 
+
 def cache(compute_key, container_factory):
     marker = object()
+
     def decorator(func):
         def replacement(*args, **kwargs):
             cache = container_factory()
@@ -74,23 +96,28 @@ def cache(compute_key, container_factory):
         return replacement
     return decorator
 
+
 def request_cache(compute_key):
     return cache(compute_key, request_container)
 
+
 class LRUCacheSetItem(LRUCache):
     __setitem__ = LRUCache.put
 
 _lru_cache = LRUCacheSetItem(1000)
 
+
 def lru_cache(compute_key):
     return cache(compute_key, lambda: _lru_cache)
 
-def clear_cache(): # only useful for tests really
+
+def clear_cache():  # only useful for tests really
     request = get_current_request()
     if request is not None:
         setattr(request, _CACHE_ATTR, None)
     _lru_cache.clear()
 
+
 def extract_from_settings(prefix, settings=None):
     """
       >>> settings = {
@@ -106,6 +133,7 @@ def extract_from_settings(prefix, settings=None):
             extracted[key[len(prefix):]] = value
     return extracted
 
+
 def disambiguate_name(name):
     parts = name.split(u'-')
     if len(parts) > 1:
@@ -119,17 +147,21 @@ def disambiguate_name(name):
         parts.append(u'1')
     return u'-'.join(parts)
 
+
 def title_to_name(title, blacklist=()):
     request = get_current_request()
     if request is not None:
         locale_name = get_locale_name(request)
     else:
         locale_name = 'en'
-    name = unicode(urlnormalizer.normalize(title, locale_name, max_length=40))
+    from kotti import get_settings
+    urlnormalizer = get_settings()['kotti.url_normalizer'][0]
+    name = unicode(urlnormalizer(title, locale_name, max_length=40))
     while name in blacklist:
         name = disambiguate_name(name)
     return name
 
+
 def camel_case_to_name(text):
     """
       >>> camel_case_to_name('FooBar')
@@ -147,6 +179,16 @@ def camel_case_to_name(text):
         r'((?<=[a-z])[A-Z]|(?<!\A)[A-Z](?=[a-z]))', r'_\1', text).lower()
 
 
+def command(func, doc):
+    args = docopt(doc)
+    pyramid_env = bootstrap(args['<config_uri>'])
+    try:
+        func(args)
+    finally:
+        pyramid_env['closer']()
+    return 0
+
+
 from kotti.sqla import JsonType
 from kotti.sqla import MutationDict
 from kotti.sqla import MutationList
@@ -155,8 +197,9 @@ from kotti.sqla import NestedMutationDict
 from kotti.sqla import NestedMutationList
 
 
-for name in ('JsonType', 'MutationDict', 'MutationList', 'NestedMixin',
-             'NestedMutationDict', 'NestedMutationList'):
+for cls in (JsonType, MutationDict, MutationList, NestedMixin,
+            NestedMutationDict, NestedMutationList):
+    name = cls.__name__
     deprecated(
         name,
         "kotti.util.{0} has been moved to the kotti.sqla "
