@@ -6,11 +6,11 @@ A simple example that'll include the output of the 'hello_world' view
 in in the left column of every page::
 
   from kotti.views.slots import assign_slot
-  assign_slot('hello_world', 'left_slot')
+  assign_slot('hello_world', 'left')
 
 It is also possible to pass parameters to the view:
 
-  assign_slot('last_tweets', 'right_slot', params=dict(user='foo'))
+  assign_slot('last_tweets', 'right', params=dict(user='foo'))
 
 If no view can be found for the given request and slot, the slot
 remains empty.
@@ -20,10 +20,12 @@ inside an ``includeme`` function and not on a module level, to allow
 users of your package to include your slot assignments through the
 ``pyramid.includes`` configuration setting.  """
 
+import urllib
+
 from pyramid.exceptions import PredicateMismatch
 from pyramid.request import Request
 from pyramid.view import render_view
-import urllib
+from pyramid.view import view_config
 from zope.deprecation import deprecate
 
 from kotti.events import ObjectEvent
@@ -75,6 +77,10 @@ def _render_view_on_slot_event(view_name, event, params):
         POST=_encode(params),
         )
 
+    post_items = request.POST.items()
+    if post_items:
+        view_request.POST.extend(post_items)
+
     # This is quite brittle:
     for name in REQUEST_ATTRS_TO_COPY:
         setattr(view_request, name, getattr(request, name))
@@ -143,6 +149,8 @@ slot_events = [
     ]
 
 
+@view_config(name='local-navigation',
+             renderer='kotti:templates/view/nav-local.pt')
 def local_navigation(context, request):
     from kotti.resources import get_root
 
@@ -162,8 +170,5 @@ def local_navigation(context, request):
 
 
 def includeme_local_navigation(config):
-    config.add_view(
-        local_navigation,
-        name='local-navigation',
-        renderer='kotti:templates/view/nav-local.pt')
+    config.scan(__name__)
     assign_slot('local-navigation', 'right')
