@@ -4,35 +4,41 @@
 import re
 from urllib import urlencode
 
-from pyramid.httpexceptions import HTTPFound
-from pyramid.exceptions import Forbidden
 import colander
-import deform
 from deform import Button
+from deform import Set
 from deform.widget import AutocompleteInputWidget
-from deform.widget import CheckedPasswordWidget
 from deform.widget import CheckboxChoiceWidget
+from deform.widget import CheckedPasswordWidget
 from deform.widget import SequenceWidget
+from pyramid.exceptions import Forbidden
+from pyramid.httpexceptions import HTTPFound
+from pyramid.view import view_config
+from pyramid_deform import FormView
 
-from kotti.message import send_set_password
-from kotti.security import USER_MANAGEMENT_ROLES
+from kotti.events import UserDeleted
+from kotti.events import notify
+from kotti.message import email_set_password
+from kotti.resources import get_root
 from kotti.security import ROLES
 from kotti.security import SHARING_ROLES
+from kotti.security import USER_MANAGEMENT_ROLES
 from kotti.security import get_principals
-from kotti.security import map_principals_with_local_roles
-from kotti.security import list_groups_raw
 from kotti.security import list_groups_ext
+from kotti.security import list_groups_raw
+from kotti.security import map_principals_with_local_roles
 from kotti.security import set_groups
 from kotti.util import _
+from kotti.views.form import AddFormView
+from kotti.views.form import EditFormView
 from kotti.views.site_setup import CONTROL_PANEL_LINKS
-from kotti.views.util import template_api
 from kotti.views.util import is_root
-from kotti.views.util import AddFormView
-from kotti.views.util import EditFormView
+from kotti.views.util import template_api
+
 
 def roles_form_handler(context, request, available_role_names, groups_lister):
     changed = []
-    
+
     if 'apply' in request.POST:
         p_to_r = {}
         for name in request.params:
@@ -68,6 +74,7 @@ def roles_form_handler(context, request, available_role_names, groups_lister):
 
     return changed
 
+
 def search_principals(request, context=None, ignore=None, extra=()):
     flash = request.session.flash
     principals = get_principals()
@@ -94,6 +101,9 @@ def search_principals(request, context=None, ignore=None, extra=()):
 
     return entries
 
+
+@view_config(name='share', permission='manage',
+             renderer='kotti:templates/edit/share.pt')
 def share_node(context, request):
     # Allow roles_form_handler to do processing on 'apply':
     changed = roles_form_handler(
@@ -104,9 +114,11 @@ def share_node(context, request):
         return HTTPFound(location=request.url)
 
     existing = map_principals_with_local_roles(context)
+
     def with_roles(entry):
         all_groups = entry[1][0]
         return [g for g in all_groups if g.startswith('role:')]
+
     existing = filter(with_roles, existing)
     seen = set([entry[0].name for entry in existing])
 
@@ -119,6 +131,7 @@ def share_node(context, request):
         'available_roles': available_roles,
         }
 
+
 def name_pattern_validator(node, value):
     """
       >>> name_pattern_validator(None, u'bob')
@@ -133,21 +146,44 @@ def name_pattern_validator(node, value):
     if not valid_pattern.match(value):
         raise colander.Invalid(node, _(u"Invalid value"))
 
+
 def name_new_validator(node, value):
     if get_principals().get(value.lower()) is not None:
         raise colander.Invalid(
             node, _(u"A user with that name already exists."))
 
+
+@colander.deferred
+def deferred_email_validator(node, kw):
+    def raise_invalid_email(node, value):
+        raise colander.Invalid(
+            node, _(u"A user with that email already exists."))
+    request = kw['request']
+    if request.POST:
+        email = request.params.get('email')
+        name = request.params.get('name')
+        if not name and request.user:
+            name = request.user.name
+        if email and name:
+            principals = get_principals()
+            if any(p for p in principals.search(email=email)
+                   if p.name.lower() != name.lower()):
+                # verify duplicated email except myself when update info
+                return raise_invalid_email
+
+
 def roleset_validator(node, value):
     oneof = colander.OneOf(USER_MANAGEMENT_ROLES)
     [oneof(node, item) for item in value]
 
+
 def group_validator(node, value):
     principals = get_principals()
     if principals.get('group:' + value) is None:
         raise colander.Invalid(node, _(u"No such group: ${group}",
                                        mapping=dict(group=value)))
 
+
 class Groups(colander.SequenceSchema):
     group = colander.SchemaNode(
         colander.String(),
@@ -157,9 +193,15 @@ class Groups(colander.SequenceSchema):
         widget=AutocompleteInputWidget(),
         )
 
+
 class PrincipalBasic(colander.MappingSchema):
     title = colander.SchemaNode(colander.String(), title=_(u'Title'))
-    email = colander.SchemaNode(colander.String(), title=_(u'Email'))
+    email = colander.SchemaNode(
+        colander.String(),
+        title=_(u'Email'),
+        validator=deferred_email_validator,
+    )
+
 
 class PrincipalFull(PrincipalBasic):
     name = colander.SchemaNode(
@@ -180,7 +222,7 @@ class PrincipalFull(PrincipalBasic):
         description=_(u"Untick this to deactivate the account."),
         )
     roles = colander.SchemaNode(
-        deform.Set(allow_empty=True),
+        Set(allow_empty=True),
         validator=roleset_validator,
         missing=[],
         title=_(u"Global roles"),
@@ -195,6 +237,7 @@ class PrincipalFull(PrincipalBasic):
         widget=SequenceWidget(min_len=1),
         )
 
+
 def principal_schema(base=PrincipalFull()):
     principals = get_principals()
     schema = base.clone()
@@ -214,6 +257,7 @@ def principal_schema(base=PrincipalFull()):
             (n, ROLES[n].title) for n in USER_MANAGEMENT_ROLES]
     return schema
 
+
 def user_schema(base=PrincipalFull()):
     schema = principal_schema(base)
     has_password = True
@@ -223,16 +267,20 @@ def user_schema(base=PrincipalFull()):
         has_password = False
     if has_password:
         schema['password'].description = _(
-            u"Leave this empty and tick the 'Send password registration' box below to have the user set their own password.")
+            u"Leave this empty and tick the 'Send password registration' "
+            "box below to have the user set their own password."
+            )
     schema['title'].title = _(u"Full name")
     return schema
 
+
 def group_schema(base=PrincipalFull()):
     schema = principal_schema(base)
     del schema['password']
     schema['email'].missing = None
     return schema
 
+
 def _massage_groups_in(appstruct):
     """Manipulate appstruct received from form so that it's suitable
     for saving on the Principal.
@@ -249,20 +297,23 @@ def _massage_groups_in(appstruct):
     del appstruct['roles']
     appstruct['groups'] = all_groups
 
+
 def _massage_groups_out(appstruct):
     """Opposite of '_massage_groups_in': remove 'groups:' prefix and
     split 'groups' into 'roles' and 'groups'.
     """
     d = appstruct
     groups = [g.split(u'group:')[1] for g in d['groups']
-              if g.startswith(u'group:')]
-    roles = [r for r in d['groups'] if r.startswith(u'role:')]
+              if g and g.startswith(u'group:')]
+    roles = [r for r in d['groups'] if r and r.startswith(u'role:')]
     d['groups'] = groups
     d['roles'] = roles
     return d
 
+
 class UserAddFormView(AddFormView):
     item_type = _(u'User')
+
     buttons = (Button('add_user', _(u'Add User')),
                Button('cancel', _(u'Cancel')))
 
@@ -285,14 +336,14 @@ class UserAddFormView(AddFormView):
         send_email = appstruct.pop('send_email', False)
         get_principals()[name] = appstruct
         if send_email:
-            send_set_password(get_principals()[name], self.request)
+            email_set_password(get_principals()[name], self.request)
         self.request.session.flash(_(u'${title} added.',
-                                     mapping=dict(title=appstruct['title'])),
-                                     'success')
+            mapping=dict(title=appstruct['title'])), 'success')
         location = self.request.url.split('?')[0] + '?' + urlencode(
             {'extra': name})
         return HTTPFound(location=location)
 
+
 class GroupAddFormView(UserAddFormView):
     buttons = (Button('add_group', _(u'Add Group')),
                Button('cancel', _(u'Cancel')))
@@ -306,54 +357,69 @@ class GroupAddFormView(UserAddFormView):
         appstruct['name'] = u'group:%s' % appstruct['name'].lower()
         return self.add_user_success(appstruct)
 
-def users_manage(context, request):
-    api = template_api(
-        context, request,
-        cp_links=CONTROL_PANEL_LINKS,
-        )
-    api.page_title = _(u"User Management - ${title}",
-                       mapping=dict(title=api.site_title))
 
-    principals = get_principals()
+@view_config(name='setup-users', permission='admin',
+             custom_predicates=(is_root,),
+             renderer='kotti:templates/site-setup/users.pt')
+class UsersManage(FormView):
 
-    def groups_lister(principal_name, context):
-        return principals[principal_name].groups
+    UserAddFormView = UserAddFormView
+    GroupAddFormView = GroupAddFormView
 
-    # Handling the user/roles matrix:
-    changed = roles_form_handler(
-        context, request, USER_MANAGEMENT_ROLES, groups_lister)
-    if changed:
-        changed_names = []
-        for (principal_name, context, groups) in changed:
-            principal = principals[principal_name]
-            principal.groups = list(groups)
-            changed_names.append(principal_name)
-        location = request.url.split('?')[0] + '?' + urlencode(
-            {'extra': ','.join(changed_names)})
-        return HTTPFound(location=location)
+    def __init__(self, context, request):
+        self.context = context
+        self.request = request
 
-    extra = request.params.get('extra') or ()
-    if extra:
-        extra = extra.split(',')
-    search_entries = search_principals(request, extra=extra)
-    available_roles = [ROLES[role_name] for role_name in USER_MANAGEMENT_ROLES]
+    def __call__(self):
+        api = template_api(
+            self.context, self.request,
+            cp_links=CONTROL_PANEL_LINKS,
+            )
+        api.page_title = _(u"User Management - ${title}",
+                           mapping=dict(title=api.site_title))
 
-    # Add forms:
-    user_addform = UserAddFormView(context, request)()
-    if request.is_response(user_addform):
-        return user_addform
+        principals = get_principals()
 
-    group_addform = GroupAddFormView(context, request)()
-    if request.is_response(group_addform):
-        return group_addform
+        def groups_lister(principal_name, context):
+            return principals[principal_name].groups
+
+        # Handling the user/roles matrix:
+        changed = roles_form_handler(
+            self.context, self.request, USER_MANAGEMENT_ROLES, groups_lister)
+        if changed:
+            changed_names = []
+            for (principal_name, context, groups) in changed:
+                principal = principals[principal_name]
+                principal.groups = list(groups)
+                changed_names.append(principal_name)
+            location = self.request.url.split('?')[0] + '?' + urlencode(
+                {'extra': ','.join(changed_names)})
+            return HTTPFound(location=location)
+
+        extra = self.request.params.get('extra') or ()
+        if extra:
+            extra = extra.split(',')
+        search_entries = search_principals(self.request, extra=extra)
+        available_roles = [ROLES[role_name]
+                           for role_name in USER_MANAGEMENT_ROLES]
+
+        # Add forms:
+        user_addform = self.UserAddFormView(self.context, self.request)()
+        if self.request.is_response(user_addform):
+            return user_addform
+
+        group_addform = self.GroupAddFormView(self.context, self.request)()
+        if self.request.is_response(group_addform):
+            return group_addform
+
+        return {
+            'api': api,
+            'entries': search_entries,
+            'available_roles': available_roles,
+            'user_addform': user_addform['form'],
+            'group_addform': group_addform['form'],
+            }
 
-    return {
-        'api': api,
-        'entries': search_entries,
-        'available_roles': available_roles,
-        'user_addform': user_addform['form'],
-        'group_addform': group_addform['form'],
-        }
 
 class UserEditFormView(EditFormView):
     @property
@@ -363,7 +429,13 @@ class UserEditFormView(EditFormView):
     def schema_factory(self):
         return user_schema(PrincipalBasic())
 
+
 class UserManageFormView(UserEditFormView):
+
+    buttons = (Button('save', _(u'Save')),
+               Button('cancel', _(u'Cancel')),
+               Button('delete', _(u'Delete')))
+
     def schema_factory(self):
         schema = user_schema()
         del schema['name']
@@ -377,70 +449,144 @@ class UserManageFormView(UserEditFormView):
         _massage_groups_in(appstruct)
         return super(UserEditFormView, self).save_success(appstruct)
 
-def user_manage(context, request):
-    username = request.params['name']
-    principal = get_principals()[username]
+    def cancel_success(self, appstruct):
+        self.request.session.flash(_(u'No changes made.'), 'info')
+        location = "%s/@@setup-users" % self.request.application_url
+        return HTTPFound(location=location)
+    cancel_failure = cancel_success
 
-    api = template_api(
-        context, request,
-        page_title=_(u"Edit User - ${title}",
-                     mapping=dict(title=context.title)),
-        cp_links=CONTROL_PANEL_LINKS,
-        principal=principal,
-        )
+    def delete_success(self, appstruct):
+        location = "%s/@@delete-user?name=%s" % (
+            self.request.application_url, self.request.params['name'])
+        return HTTPFound(location=location)
 
-    form = UserManageFormView(principal, request)()
-    if request.is_response(form):
-        return form
 
-    return {
-        'api': api,
-        'form': form['form'],
-        }
+class GroupManageFormView(UserManageFormView):
+    def schema_factory(self):
+        schema = group_schema()
+        del schema['name']
+        del schema['active']
+        return schema
 
-def preferences(context, request):
-    api = template_api(context, request)
-    api.page_title = _(u"My preferences - ${title}",
-                       mapping=dict(title=api.site_title))
-    user = api.user
 
-    form = UserEditFormView(user, request)()
-    if request.is_response(form):
-        return form
+@view_config(name='setup-user', permission='admin',
+             custom_predicates=(is_root, ),
+             renderer='kotti:templates/site-setup/user.pt')
+class UserManage(FormView):
+
+    GroupManageFormView = GroupManageFormView
+    UserManageFormView = UserManageFormView
+
+    def __init__(self, context, request):
+        self.context = context
+        self.request = request
+
+    def __call__(self):
+        user_or_group = self.request.params['name']
+        principal = get_principals()[user_or_group]
+
+        is_group = user_or_group.startswith("group:")
+        principal_type = _(u"Group") if is_group else _(u"User")
+
+        api = template_api(
+            self.context, self.request,
+            page_title=_(u"Edit ${principal_type} - ${title}",
+                         mapping=dict(principal_type=principal_type,
+                                      title=self.context.title)),
+            cp_links=CONTROL_PANEL_LINKS,
+            principal=principal,
+            )
+
+        form_view = self.GroupManageFormView if is_group \
+            else self.UserManageFormView
+        form = form_view(principal, self.request)()
+        if self.request.is_response(form):
+            return form
+
+        return {
+            'api': api,
+            'form': form['form'],
+            }
+
+
+@view_config(name='delete-user', permission='admin',
+             custom_predicates=(is_root,),
+             renderer='kotti:templates/site-setup/delete-user.pt')
+def user_delete(context, request):
+    principals = get_principals()
+
+    if 'name' in request.params and request.params['name']:
+        user_or_group = request.params['name']
+        principal = principals.search(name=user_or_group).first()
+        if principal is None:
+            request.session.flash(_(u'User not found.'), 'error')
+        else:
+            is_group = user_or_group.startswith("group:")
+            principal_type = _(u"Group") if is_group else _(u"User")
+
+            # We already coming from the confirmation page.
+            if 'delete' in request.POST:
+                principals.__delitem__(principal.name)
+                notify(UserDeleted(principal, request))
+                request.session.flash(_(u'${principal_type} ${title} deleted.',
+                                        mapping=dict(principal_type=principal_type,
+                                                     title=principal.title)), 'info')
+                location = "%s/@@setup-users" % request.application_url
+                return HTTPFound(location=location)
+
+            api = template_api(
+                context, request,
+                page_title=_(u"Delete ${principal_type} - ${title}",
+                             mapping=dict(principal_type=principal_type,
+                                          title=principal.title)),
+                principal_type=principal_type,
+                principal=principal,
+                )
+            return {'api': api, }
+    else:
+        request.session.flash(_(u'No name given.'), 'error')
+
+    return {'api': template_api(context, request), }
+
+
+class PreferencesFormView(UserEditFormView):
+
+    def cancel_success(self, appstruct):
+        location = self.request.resource_url(get_root())
+        return HTTPFound(location=location)
+    cancel_failure = cancel_success
+
+
+@view_config(name='prefs', custom_predicates=(is_root, ),
+             renderer='kotti:templates/edit/simpleform.pt')
+class Preferences(FormView):
+
+    PreferencesFormView = PreferencesFormView
+
+    def __init__(self, context, request):
+        self.context = context
+        self.request = request
+
+    def __call__(self):
+        user = self.request.user
+        if user is None:
+            raise Forbidden()
+
+        api = template_api(self.context, self.request)
+        api.page_title = _(u"My preferences - ${title}",
+                           mapping=dict(title=api.site_title))
+
+        form = self.PreferencesFormView(user, self.request)()
+
+        if self.request.is_response(form):
+            return form
+
+        return {
+            'api': api,
+            'form': form['form'],
+            'macro': api.macro('kotti:templates/site-setup/master.pt'),
+            }
 
-    return {
-        'api': api,
-        'form': form['form'],
-        'macro': api.macro('kotti:templates/site-setup/master.pt'),
-        }
 
 def includeme(config):
-    config.add_view(
-        share_node,
-        name='share',
-        permission='manage',
-        renderer='kotti:templates/edit/share.pt',
-        )
-
-    config.add_view(
-        users_manage,
-        name='setup-users',
-        permission='admin',
-        custom_predicates=(is_root,),
-        renderer='kotti:templates/site-setup/users.pt',
-        )
-
-    config.add_view(
-        user_manage,
-        name='setup-user',
-        permission='admin',
-        custom_predicates=(is_root,),
-        renderer='kotti:templates/site-setup/user.pt',
-        )
-
-    config.add_view(
-        preferences,
-        name='prefs',
-        custom_predicates=(is_root,),
-        renderer='kotti:templates/edit/simpleform.pt',
-        )
+    config.scan(__name__)
